# pytzer: Pitzer model for chemical activities in aqueous solutions
# Copyright (C) 2019  Matthew Paul Humphreys  (GNU GPLv3)

"""Specify tabulated Pitzer model coefficient values."""

# The first set of tables from Pitzer (1991) Chapter 3 are all just
#  reproductions of the values listed by Silvester and Pitzer (1979).

# Pitzer (1991), Chapter 3, Table 12 [p109-p110]
# First temperature derivatives of b0, b1 and Cphi for 1-1 electrolytes
# 'max' gives maximum applicable molality in mol/kg
# INCOMPLETE: only includes values for H, Li, Na, K cations for now
P91_Ch3_T12 = {
    'info': 'First temperature derivatives of b0, b1 and Cphi ' \
          + 'for 1-1 electrolytes - INCOMPLETE',
    'H-Cl'   : {'b0': - 3.081e-4, 'b1':   1.419e-4, 'Cphi': - 6.213e-5, 'max': 4.5},
    'H-Br'   : {'b0': - 2.049e-4, 'b1':   4.467e-4, 'Cphi': - 5.685e-5, 'max': 6.0},
    'H-I'    : {'b0': - 0.230e-4, 'b1':   8.860e-4, 'Cphi': - 7.320e-5, 'max': 6.0},
    'H-ClO4' : {'b0':   4.905e-4, 'b1':  19.310e-4, 'Cphi': -11.770e-5, 'max': 6.0},
    'Li-Cl'  : {'b0': - 1.685e-4, 'b1':   5.366e-4, 'Cphi': - 4.520e-5, 'max': 6.4},
    'Li-Br'  : {'b0': - 1.891e-4, 'b1':   6.636e-4, 'Cphi': - 2.813e-5, 'max': 6.0},
    'Li-ClO4': {'b0':   0.386e-4, 'b1':   7.009e-4, 'Cphi': - 7.712e-5, 'max': 4.0},
    'Na-F'   : {'b0':   5.361e-4, 'b1':   8.700e-4, 'Cphi':   0       , 'max': 0.7},
    'Na-Cl'  : {'b0':   7.159e-4, 'b1':   7.005e-4, 'Cphi': -10.540e-5, 'max': 6.0},
    'Na-Br'  : {'b0':   7.692e-4, 'b1':  10.790e-4, 'Cphi': - 9.300e-5, 'max': 9.0},
    'Na-I'   : {'b0':   8.355e-4, 'b1':   8.280e-4, 'Cphi': - 8.350e-5, 'max': 6.0},
    'Na-OH'  : {'b0':   7.000e-4, 'b1':   1.340e-4, 'Cphi': -18.940e-5, 'max': 4.2},
    'Na-ClO3': {'b0':  10.350e-4, 'b1':  19.070e-4, 'Cphi': - 9.290e-5, 'max': 6.4},
    'Na-ClO4': {'b0':  12.960e-4, 'b1':  22.970e-4, 'Cphi': -16.230e-5, 'max': 6.0},
    'Na-BrO3': {'b0':   5.590e-4, 'b1':  34.370e-4, 'Cphi':   0       , 'max': 0.1},
    'Na-IO3' : {'b0':  20.660e-4, 'b1':  60.570e-4, 'Cphi':   0       , 'max': 0.1},
    'Na-SCN' : {'b0':   7.800e-4, 'b1':  20.000e-4, 'Cphi':   0       , 'max': 0.1},
    'Na-NO3' : {'b0':  12.660e-4, 'b1':  20.600e-4, 'Cphi': -23.160e-5, 'max': 2.2},
    'K-F'    : {'b0':   2.140e-4, 'b1':   5.440e-4, 'Cphi': - 5.950e-5, 'max': 5.9},
    'K-Cl'   : {'b0':   5.794e-4, 'b1':  10.710e-4, 'Cphi': - 5.095e-5, 'max': 4.5},
    'K-Br'   : {'b0':   7.390e-4, 'b1':  17.400e-4, 'Cphi': - 7.004e-5, 'max': 5.2},
    'K-I'    : {'b0':   9.914e-4, 'b1':  11.860e-4, 'Cphi': - 9.440e-5, 'max': 7.0},
    'K-ClO3' : {'b0':  19.870e-4, 'b1':  31.800e-4, 'Cphi':   0       , 'max': 0.1},
    'K-ClO4' : {'b0':   0.600e-4, 'b1': 100.700e-4, 'Cphi':   0       , 'max': 0.1},
    'K-SCN'  : {'b0':   6.870e-4, 'b1':  37.000e-4, 'Cphi':   0.430e-5, 'max': 3.1},
    'K-NO3'  : {'b0':   2.060e-4, 'b1':  64.500e-4, 'Cphi':  39.700e-5, 'max': 2.4},
    'K-H2PO4': {'b0':   6.045e-4, 'b1':  28.600e-4, 'Cphi': -10.110e-5, 'max': 1.8}}

# Pitzer (1991), Chapter 3, Table 13, Part I [p111]
# First temperature derivatives of b0, b1 and C0 for 2-1 and 1-2 electrolytes
# 'max' gives maximum applicable molality in mol/kg
P91_Ch3_T13_I = {
    'info': 'First temperature derivatives of b0, b1 and C0 ' \
          + 'for 2-1 and 1-2 electrolytes',
    'Mg-Cl'    : {'b0': -0.194e-3, 'b1':  2.78e-3, 'C0': -0.580e-4, 'max': 2.0},
    'Mg-Br'    : {'b0': -0.056e-3, 'b1':  3.86e-3, 'C0':  0       , 'max': 0.1},
    'Mg-ClO4'  : {'b0':  0.523e-3, 'b1':  4.50e-3, 'C0': -1.250e-4, 'max': 3.2},
    'Mg-NO3'   : {'b0':  0.515e-3, 'b1':  4.49e-3, 'C0':  0       , 'max': 0.1},
    'Ca-Cl'    : {'b0': -0.561e-3, 'b1':  2.66e-3, 'C0': -0.723e-4, 'max': 6.0},
    'Ca-Br'    : {'b0': -0.523e-3, 'b1':  6.04e-3, 'C0':  0       , 'max': 0.6},
    'Ca-NO3'   : {'b0':  0.530e-3, 'b1':  9.19e-3, 'C0':  0       , 'max': 0.1},
    'Ca-ClO4'  : {'b0':  0.830e-3, 'b1':  5.08e-3, 'C0': -1.090e-4, 'max': 4.0},
    'Sr-Cl'    : {'b0':  0.717e-3, 'b1':  2.84e-3, 'C0':  0       , 'max': 0.1},
    'Sr-Br'    : {'b0': -0.328e-3, 'b1':  6.53e-3, 'C0':  0       , 'max': 0.1},
    'Sr-NO3'   : {'b0':  0.177e-3, 'b1': 12.47e-3, 'C0':  0       , 'max': 0.2},
    'Sr-ClO4'  : {'b0':  1.143e-3, 'b1':  5.39e-3, 'C0': -1.100e-4, 'max': 3.0},
    'Ba-Cl'    : {'b0':  0.640e-3, 'b1':  3.23e-3, 'C0': -0.540e-4, 'max': 1.8},
    'Ba-Br'    : {'b0': -0.338e-3, 'b1':  6.78e-3, 'C0':  0       , 'max': 0.1},
    'Ba-NO3'   : {'b0': -2.910e-3, 'b1': 29.10e-3, 'C0':  0       , 'max': 0.1},
    'Mnjj-ClO4': {'b0':  0.397e-3, 'b1':  5.03e-3, 'C0': -1.180e-4, 'max': 4.0},
    'Cojj-ClO4': {'b0':  0.545e-3, 'b1':  5.36e-3, 'C0': -1.270e-4, 'max': 4.0},
    'Nijj-ClO4': {'b0':  0.666e-3, 'b1':  4.76e-3, 'C0': -1.350e-4, 'max': 4.0},
    'Cujj-Cl'  : {'b0': -2.710e-3, 'b1':  8.50e-3, 'C0':  0       , 'max': 0.6},
    'Znjj-ClO4': {'b0':  0.596e-3, 'b1':  5.09e-3, 'C0': -1.360e-4, 'max': 4.0},
    'Li-SO4'   : {'b0':  0.506e-3, 'b1':  1.41e-3, 'C0': -0.825e-4, 'max': 3.0},
    'Na-SO4'   : {'b0':  2.367e-3, 'b1':  5.63e-3, 'C0': -1.725e-4, 'max': 3.0},
    'K-SO4'    : {'b0':  1.440e-3, 'b1':  6.70e-3, 'C0':  0       , 'max': 0.1},
    'Rb-SO4'   : {'b0':  0.940e-3, 'b1':  8.64e-3, 'C0':  0       , 'max': 0.1},
    'Cs-SO4'   : {'b0': -0.893e-3, 'b1': 14.48e-3, 'C0':  0       , 'max': 0.1}}

# Pitzer (1991), Chapter 3, Table 13, Part II [p111]
# First temperature derivatives of b0, b1, b2 and C0 for 3-1 and 2-2 electrolytes
# 'max' gives maximum applicable molality in mol/kg
# alph1 = 1.4 for the M-SO4 b1's, alph1 = 2.0 otherwise
# alph2 = 12.0 throughout
#
# Na3-FeCN5 is a typo for Na3-FeCN6
P91_Ch3_T13_II = {
    'info': 'First temperature derivatives of b0, b1, b2 and C0 ' \
          + 'for 3-1 and 2-2 electrolytes',
    'La-Cl'      : {'b0':  0.253e-3, 'b1': 0.798e-2, 'b2':  0      , 'C0': -0.107e-3, 'max': 3.6 },
    'La-ClO4'    : {'b0':  0.152e-3, 'b1': 1.503e-2, 'b2':  0      , 'C0': -0.194e-3, 'max': 2.1 },
    'La-NO3'     : {'b0':  0.173e-3, 'b1': 1.095e-2, 'b2':  0      , 'C0': -0.130e-3, 'max': 2.2 },
    'Na-FejjjCN6': {'b0':  3.050e-3, 'b1': 1.520e-2, 'b2':  0      , 'C0':  0       , 'max': 0.1 },
    'K-FejjjCN6' : {'b0': -0.870e-3, 'b1': 3.150e-2, 'b2':  0      , 'C0':  0       , 'max': 0.1 },
    'K-FejjCN6'  : {'b0':  4.740e-3, 'b1': 3.920e-2, 'b2':  0      , 'C0':  0       , 'max': 0.2 },
    'Mg-SO4'     : {'b0': -0.690e-3, 'b1': 1.530e-2, 'b2': -2.53e-1, 'C0':  0.131e-3, 'max': 2.0 },
    'Ca-SO4'     : {'b0':  0       , 'b1': 5.460e-2, 'b2': -5.16e-1, 'C0':  0       , 'max': 0.02},
    'Cu-SO4'     : {'b0': -4.400e-3, 'b1': 2.380e-2, 'b2': -4.73e-1, 'C0':  1.200e-3, 'max': 1.0 },
    'Zn-SO4'     : {'b0': -3.660e-3, 'b1': 2.330e-2, 'b2': -3.33e-1, 'C0':  0.990e-3, 'max': 1.0 },
    'Cd-SO4'     : {'b0': -2.790e-3, 'b1': 1.710e-2, 'b2': -5.22e-1, 'C0':  0.650e-3, 'max': 1.0 }}

#======== PITZER AND MARGOYA 1973 =============================================
#
# Automatically generated with PM73_tablegen.py

PM73_TableI = {
            'H-Cl': {'b0':  0.17750, 'b1':  0.29450, 'Cphi':  0.00080},
            'H-Br': {'b0':  0.19600, 'b1':  0.35640, 'Cphi':  0.00827},
             'H-I': {'b0':  0.23620, 'b1':  0.39200, 'Cphi':  0.00110},
          'H-ClO4': {'b0':  0.17470, 'b1':  0.29310, 'Cphi':  0.00819},
           'H-NO3': {'b0':  0.11190, 'b1':  0.32060, 'Cphi':  0.00100},
           'Li-Cl': {'b0':  0.14940, 'b1':  0.30740, 'Cphi':  0.00359},
           'Li-Br': {'b0':  0.17480, 'b1':  0.25470, 'Cphi':  0.00530},
            'Li-I': {'b0':  0.21040, 'b1':  0.37300, 'Cphi':  0.00000},
           'Li-OH': {'b0':  0.01500, 'b1':  0.14000, 'Cphi':  0.00000},
         'Li-ClO4': {'b0':  0.19730, 'b1':  0.39960, 'Cphi':  0.00080},
          'Li-NO2': {'b0':  0.13360, 'b1':  0.32500, 'Cphi': -0.00530},
          'Li-NO3': {'b0':  0.14200, 'b1':  0.27800, 'Cphi': -0.00551},
            'Na-F': {'b0':  0.02150, 'b1':  0.21070, 'Cphi':  0.00000},
           'Na-Cl': {'b0':  0.07650, 'b1':  0.26640, 'Cphi':  0.00127},
           'Na-Br': {'b0':  0.09730, 'b1':  0.27910, 'Cphi':  0.00116},
            'Na-I': {'b0':  0.11950, 'b1':  0.34390, 'Cphi':  0.00180},
           'Na-OH': {'b0':  0.08640, 'b1':  0.25300, 'Cphi':  0.00440},
         'Na-ClO3': {'b0':  0.02490, 'b1':  0.24550, 'Cphi':  0.00040},
         'Na-ClO4': {'b0':  0.05540, 'b1':  0.27550, 'Cphi': -0.00118},
         'Na-BrO3': {'b0': -0.02050, 'b1':  0.19100, 'Cphi':  0.00590},
          'Na-CNS': {'b0':  0.10050, 'b1':  0.35820, 'Cphi': -0.00303},
          'Na-NO2': {'b0':  0.06410, 'b1':  0.10150, 'Cphi': -0.00490},
          'Na-NO3': {'b0':  0.00680, 'b1':  0.17830, 'Cphi': -0.00072},
        'Na-H2PO4': {'b0': -0.05330, 'b1':  0.03960, 'Cphi':  0.00795},
       'Na-H2AsO4': {'b0': -0.04420, 'b1':  0.28950, 'Cphi':  0.00000},
          'Na-BO2': {'b0': -0.05260, 'b1':  0.11040, 'Cphi':  0.01540},
          'Na-BF4': {'b0': -0.02520, 'b1':  0.18240, 'Cphi':  0.00210},
             'K-F': {'b0':  0.08089, 'b1':  0.20210, 'Cphi':  0.00093},
            'K-Cl': {'b0':  0.04835, 'b1':  0.21220, 'Cphi': -0.00084},
            'K-Br': {'b0':  0.05690, 'b1':  0.22120, 'Cphi': -0.00180},
             'K-I': {'b0':  0.07460, 'b1':  0.25170, 'Cphi': -0.00414},
            'K-OH': {'b0':  0.12980, 'b1':  0.32000, 'Cphi':  0.00410},
          'K-ClO3': {'b0': -0.09600, 'b1':  0.24810, 'Cphi':  0.00000},
          'K-BrO3': {'b0': -0.12900, 'b1':  0.25650, 'Cphi':  0.00000},
           'K-CNS': {'b0':  0.04160, 'b1':  0.23020, 'Cphi': -0.00252},
           'K-NO2': {'b0':  0.01510, 'b1':  0.01500, 'Cphi':  0.00070},
           'K-NO3': {'b0': -0.08160, 'b1':  0.04940, 'Cphi':  0.00660},
         'K-H2PO4': {'b0': -0.06780, 'b1': -0.10420, 'Cphi':  0.00000},
        'K-H2AsO4': {'b0': -0.05840, 'b1':  0.06260, 'Cphi':  0.00000},
          'K-PtF6': {'b0': -0.16300, 'b1': -0.28200, 'Cphi':  0.00000},
            'Rb-F': {'b0':  0.11410, 'b1':  0.28420, 'Cphi': -0.01050},
           'Rb-Cl': {'b0':  0.04410, 'b1':  0.14830, 'Cphi': -0.00101},
           'Rb-Br': {'b0':  0.03960, 'b1':  0.15300, 'Cphi': -0.00144},
            'Rb-I': {'b0':  0.03970, 'b1':  0.13300, 'Cphi': -0.00108},
          'Rb-NO2': {'b0':  0.02690, 'b1': -0.15530, 'Cphi': -0.00366},
          'Rb-NO3': {'b0': -0.07890, 'b1': -0.01720, 'Cphi':  0.00529},
            'Cs-F': {'b0':  0.13060, 'b1':  0.25700, 'Cphi': -0.00430},
           'Cs-Cl': {'b0':  0.03000, 'b1':  0.05580, 'Cphi':  0.00038},
           'Cs-Br': {'b0':  0.02790, 'b1':  0.01390, 'Cphi':  0.00004},
            'Cs-I': {'b0':  0.02440, 'b1':  0.02620, 'Cphi': -0.00365},
           'Cs-OH': {'b0':  0.15000, 'b1':  0.30000, 'Cphi':  0.00000},
          'Cs-NO3': {'b0': -0.07580, 'b1': -0.06690, 'Cphi':  0.00000},
          'Cs-NO2': {'b0':  0.04270, 'b1':  0.06000, 'Cphi': -0.00510},
          'Ag-NO3': {'b0': -0.08560, 'b1':  0.00250, 'Cphi':  0.00591},
         'Tl-ClO4': {'b0': -0.08700, 'b1': -0.02300, 'Cphi':  0.00000},
          'Tl-NO3': {'b0': -0.10500, 'b1': -0.37800, 'Cphi':  0.00000},
          'NH4-Cl': {'b0':  0.05220, 'b1':  0.19180, 'Cphi': -0.00301},
          'NH4-Br': {'b0':  0.06240, 'b1':  0.19470, 'Cphi': -0.00436},
        'NH4-ClO4': {'b0': -0.01030, 'b1': -0.01940, 'Cphi':  0.00000},
         'NH4-NO3': {'b0': -0.01540, 'b1':  0.11200, 'Cphi': -0.00003},
    }

PM73_TableVI = {
           'Mg-Cl': {'b0':  0.46980 * 3/4, 'b1':  2.24200 * 3/4, 'Cphi':  0.00979 * 3/2**(3/2)},
           'Mg-Br': {'b0':  0.57690 * 3/4, 'b1':  2.33700 * 3/4, 'Cphi':  0.00589 * 3/2**(3/2)},
            'Mg-I': {'b0':  0.65360 * 3/4, 'b1':  2.40550 * 3/4, 'Cphi':  0.01496 * 3/2**(3/2)},
         'Mg-ClO4': {'b0':  0.66150 * 3/4, 'b1':  2.67800 * 3/4, 'Cphi':  0.01806 * 3/2**(3/2)},
          'Mg-NO3': {'b0':  0.48950 * 3/4, 'b1':  2.11300 * 3/4, 'Cphi': -0.03889 * 3/2**(3/2)},
           'Ca-Cl': {'b0':  0.42120 * 3/4, 'b1':  2.15200 * 3/4, 'Cphi': -0.00064 * 3/2**(3/2)},
           'Ca-Br': {'b0':  0.50880 * 3/4, 'b1':  2.15100 * 3/4, 'Cphi': -0.00485 * 3/2**(3/2)},
            'Ca-I': {'b0':  0.58390 * 3/4, 'b1':  2.40900 * 3/4, 'Cphi': -0.00158 * 3/2**(3/2)},
         'Ca-ClO4': {'b0':  0.60150 * 3/4, 'b1':  2.34200 * 3/4, 'Cphi': -0.00943 * 3/2**(3/2)},
          'Ca-NO3': {'b0':  0.28110 * 3/4, 'b1':  1.87900 * 3/4, 'Cphi': -0.03798 * 3/2**(3/2)},
           'Sr-Cl': {'b0':  0.38100 * 3/4, 'b1':  2.22300 * 3/4, 'Cphi': -0.00246 * 3/2**(3/2)},
           'Sr-Br': {'b0':  0.44150 * 3/4, 'b1':  2.28200 * 3/4, 'Cphi':  0.00231 * 3/2**(3/2)},
            'Sr-I': {'b0':  0.53500 * 3/4, 'b1':  2.48000 * 3/4, 'Cphi':  0.00501 * 3/2**(3/2)},
         'Sr-ClO4': {'b0':  0.56920 * 3/4, 'b1':  2.08900 * 3/4, 'Cphi': -0.02472 * 3/2**(3/2)},
          'Sr-NO3': {'b0':  0.17950 * 3/4, 'b1':  1.84000 * 3/4, 'Cphi': -0.03757 * 3/2**(3/2)},
           'Ba-Cl': {'b0':  0.35040 * 3/4, 'b1':  1.99500 * 3/4, 'Cphi': -0.03654 * 3/2**(3/2)},
           'Ba-Br': {'b0':  0.41940 * 3/4, 'b1':  2.09300 * 3/4, 'Cphi': -0.03009 * 3/2**(3/2)},
            'Ba-I': {'b0':  0.56250 * 3/4, 'b1':  2.24900 * 3/4, 'Cphi': -0.03286 * 3/2**(3/2)},
           'Ba-OH': {'b0':  0.22900 * 3/4, 'b1':  1.60000 * 3/4, 'Cphi':  0.00000 * 3/2**(3/2)},
         'Ba-ClO4': {'b0':  0.48190 * 3/4, 'b1':  2.10100 * 3/4, 'Cphi': -0.05894 * 3/2**(3/2)},
          'Ba-NO3': {'b0': -0.04300 * 3/4, 'b1':  1.07000 * 3/4, 'Cphi':  0.00000 * 3/2**(3/2)},
           'Mn-Cl': {'b0':  0.13630 * 3/4, 'b1':  2.06700 * 3/4, 'Cphi': -0.03865 * 3/2**(3/2)},
           'Fe-Cl': {'b0':  0.44790 * 3/4, 'b1':  2.04300 * 3/4, 'Cphi': -0.01623 * 3/2**(3/2)},
           'Co-Cl': {'b0':  0.48570 * 3/4, 'b1':  1.93600 * 3/4, 'Cphi': -0.02869 * 3/2**(3/2)},
           'Co-Br': {'b0':  0.56930 * 3/4, 'b1':  2.21300 * 3/4, 'Cphi': -0.00127 * 3/2**(3/2)},
            'Co-I': {'b0':  0.69500 * 3/4, 'b1':  2.23000 * 3/4, 'Cphi': -0.00880 * 3/2**(3/2)},
          'Co-NO3': {'b0':  0.41590 * 3/4, 'b1':  2.25400 * 3/4, 'Cphi': -0.01436 * 3/2**(3/2)},
           'Ni-Cl': {'b0':  0.46390 * 3/4, 'b1':  2.10800 * 3/4, 'Cphi': -0.00702 * 3/2**(3/2)},
           'Cu-Cl': {'b0':  0.41070 * 3/4, 'b1':  1.83500 * 3/4, 'Cphi': -0.07624 * 3/2**(3/2)},
          'Cu-NO3': {'b0':  0.42240 * 3/4, 'b1':  1.90700 * 3/4, 'Cphi': -0.04136 * 3/2**(3/2)},
           'Zn-Cl': {'b0':  0.34690 * 3/4, 'b1':  2.19000 * 3/4, 'Cphi': -0.16590 * 3/2**(3/2)},
           'Zn-Br': {'b0':  0.62130 * 3/4, 'b1':  2.17900 * 3/4, 'Cphi': -0.20350 * 3/2**(3/2)},
            'Zn-I': {'b0':  0.64280 * 3/4, 'b1':  2.59400 * 3/4, 'Cphi': -0.02690 * 3/2**(3/2)},
         'Zn-ClO4': {'b0':  0.67470 * 3/4, 'b1':  2.39600 * 3/4, 'Cphi':  0.02134 * 3/2**(3/2)},
          'Zn-NO3': {'b0':  0.46410 * 3/4, 'b1':  2.25500 * 3/4, 'Cphi': -0.02955 * 3/2**(3/2)},
          'Cd-NO3': {'b0':  0.38200 * 3/4, 'b1':  2.22400 * 3/4, 'Cphi': -0.04836 * 3/2**(3/2)},
         'Pb-ClO4': {'b0':  0.44430 * 3/4, 'b1':  2.29600 * 3/4, 'Cphi': -0.01667 * 3/2**(3/2)},
          'Pb-NO3': {'b0': -0.04820 * 3/4, 'b1':  0.38000 * 3/4, 'Cphi':  0.01005 * 3/2**(3/2)},
          'UO2-Cl': {'b0':  0.56980 * 3/4, 'b1':  2.19200 * 3/4, 'Cphi': -0.06951 * 3/2**(3/2)},
        'UO2-ClO4': {'b0':  0.81510 * 3/4, 'b1':  2.85900 * 3/4, 'Cphi':  0.04089 * 3/2**(3/2)},
         'UO2-NO3': {'b0':  0.61430 * 3/4, 'b1':  2.15100 * 3/4, 'Cphi': -0.05948 * 3/2**(3/2)},
          'Li-SO4': {'b0':  0.18170 * 3/4, 'b1':  1.69400 * 3/4, 'Cphi': -0.00753 * 3/2**(3/2)},
          'Na-SO4': {'b0':  0.02610 * 3/4, 'b1':  1.48400 * 3/4, 'Cphi':  0.00938 * 3/2**(3/2)},
         'Na-S2O3': {'b0':  0.08820 * 3/4, 'b1':  1.70100 * 3/4, 'Cphi':  0.00705 * 3/2**(3/2)},
         'Na-CrO4': {'b0':  0.12500 * 3/4, 'b1':  1.82600 * 3/4, 'Cphi': -0.00407 * 3/2**(3/2)},
          'Na-CO3': {'b0':  0.25300 * 3/4, 'b1':  1.12800 * 3/4, 'Cphi': -0.09057 * 3/2**(3/2)},
         'Na-HPO4': {'b0': -0.07770 * 3/4, 'b1':  1.95400 * 3/4, 'Cphi':  0.05540 * 3/2**(3/2)},
        'Na-HAsO4': {'b0':  0.04070 * 3/4, 'b1':  2.17300 * 3/4, 'Cphi':  0.00340 * 3/2**(3/2)},
           'K-SO4': {'b0':  0.06660 * 3/4, 'b1':  1.03900 * 3/4, 'Cphi':  0.00000 * 3/2**(3/2)},
          'K-CrO4': {'b0':  0.10110 * 3/4, 'b1':  1.65200 * 3/4, 'Cphi': -0.00147 * 3/2**(3/2)},
       'K-Pt(CN)4': {'b0':  0.08810 * 3/4, 'b1':  3.16400 * 3/4, 'Cphi':  0.02470 * 3/2**(3/2)},
          'K-HPO4': {'b0':  0.03300 * 3/4, 'b1':  1.69900 * 3/4, 'Cphi':  0.03090 * 3/2**(3/2)},
         'K-HAsO4': {'b0':  0.17280 * 3/4, 'b1':  2.19800 * 3/4, 'Cphi': -0.03360 * 3/2**(3/2)},
          'Rb-SO4': {'b0':  0.07720 * 3/4, 'b1':  1.48100 * 3/4, 'Cphi': -0.00019 * 3/2**(3/2)},
          'Cs-SO4': {'b0':  0.11840 * 3/4, 'b1':  1.48100 * 3/4, 'Cphi': -0.01131 * 3/2**(3/2)},
         'NH4-SO4': {'b0':  0.05450 * 3/4, 'b1':  0.87800 * 3/4, 'Cphi': -0.00219 * 3/2**(3/2)},
    }

PM73_TableVIII = {
           'Al-Cl': {'b0':  1.04900 * 2/3, 'b1':  8.76700 * 2/3, 'Cphi':  0.00710 * 2/3**(1/2)},
           'Sr-Cl': {'b0':  1.05000 * 2/3, 'b1':  7.97800 * 2/3, 'Cphi': -0.08400 * 2/3**(1/2)},
            'Y-Cl': {'b0':  0.95990 * 2/3, 'b1':  8.16600 * 2/3, 'Cphi': -0.05870 * 2/3**(1/2)},
           'La-Cl': {'b0':  0.91580 * 2/3, 'b1':  8.23100 * 2/3, 'Cphi': -0.08310 * 2/3**(1/2)},
           'Ce-Cl': {'b0':  0.91870 * 2/3, 'b1':  8.22700 * 2/3, 'Cphi': -0.08090 * 2/3**(1/2)},
           'Pr-Cl': {'b0':  0.90300 * 2/3, 'b1':  8.18100 * 2/3, 'Cphi': -0.07270 * 2/3**(1/2)},
           'Nd-Cl': {'b0':  0.91750 * 2/3, 'b1':  8.10400 * 2/3, 'Cphi': -0.07370 * 2/3**(1/2)},
           'Sm-Cl': {'b0':  0.93300 * 2/3, 'b1':  8.27300 * 2/3, 'Cphi': -0.07280 * 2/3**(1/2)},
           'Eu-Cl': {'b0':  0.93700 * 2/3, 'b1':  8.38500 * 2/3, 'Cphi': -0.06870 * 2/3**(1/2)},
           'Cr-Cl': {'b0':  1.10460 * 2/3, 'b1':  7.88300 * 2/3, 'Cphi': -0.11720 * 2/3**(1/2)},
          'Cr-NO3': {'b0':  1.05600 * 2/3, 'b1':  7.77700 * 2/3, 'Cphi': -0.15330 * 2/3**(1/2)},
         'Ga-ClO4': {'b0':  1.23810 * 2/3, 'b1':  9.79400 * 2/3, 'Cphi':  0.09040 * 2/3**(1/2)},
           'In-Cl': {'b0': -1.68000 * 2/3, 'b1': -3.85000 * 2/3, 'Cphi':  0.00000 * 2/3**(1/2)},
          'Na-PO4': {'b0':  0.26720 * 2/3, 'b1':  5.77700 * 2/3, 'Cphi': -0.13390 * 2/3**(1/2)},
         'Na-AsO4': {'b0':  0.35820 * 2/3, 'b1':  5.89500 * 2/3, 'Cphi': -0.12400 * 2/3**(1/2)},
           'K-PO4': {'b0':  0.55940 * 2/3, 'b1':  5.95800 * 2/3, 'Cphi': -0.22550 * 2/3**(1/2)},
          'K-P3O9': {'b0':  0.48670 * 2/3, 'b1':  8.34900 * 2/3, 'Cphi': -0.08860 * 2/3**(1/2)},
          'K-AsO4': {'b0':  0.74910 * 2/3, 'b1':  6.51100 * 2/3, 'Cphi': -0.33760 * 2/3**(1/2)},
       'K-Fe(CN)6': {'b0':  0.50350 * 2/3, 'b1':  7.12100 * 2/3, 'Cphi': -0.11760 * 2/3**(1/2)},
       'K-Co(CN)6': {'b0':  0.56030 * 2/3, 'b1':  5.81500 * 2/3, 'Cphi': -0.16030 * 2/3**(1/2)},
      'Co(en)3-Cl': {'b0':  0.26030 * 2/3, 'b1':  3.56300 * 2/3, 'Cphi': -0.09160 * 2/3**(1/2)},
     'Co(en)3-NO3': {'b0':  0.18820 * 2/3, 'b1':  3.93500 * 2/3, 'Cphi':  0.00000 * 2/3**(1/2)},
    'Co(en)3-ClO4': {'b0':  0.16190 * 2/3, 'b1':  5.39500 * 2/3, 'Cphi':  0.00000 * 2/3**(1/2)},
    'Co(pn)3-ClO4': {'b0':  0.20220 * 2/3, 'b1':  3.97600 * 2/3, 'Cphi':  0.00000 * 2/3**(1/2)},
    }

PM73_TableIX = {
           'Th-Cl': {'b0':  1.62200 * 5/8, 'b1': 21.33000 * 5/8, 'Cphi': -0.33090 * 5/16},
          'Th-NO3': {'b0':  1.54600 * 5/8, 'b1': 18.22000 * 5/8, 'Cphi': -0.59060 * 5/16},
         'Na-P2O7': {'b0':  0.69900 * 5/8, 'b1': 17.16000 * 5/8, 'Cphi':  0.00000 * 5/16},
          'K-P2O7': {'b0':  0.97700 * 5/8, 'b1': 17.88000 * 5/8, 'Cphi': -0.24180 * 5/16},
       'K-Fe(CN)6': {'b0':  1.02100 * 5/8, 'b1': 16.23000 * 5/8, 'Cphi': -0.55790 * 5/16},
       'K-Mo(CN)8': {'b0':  0.85400 * 5/8, 'b1': 18.53000 * 5/8, 'Cphi': -0.34990 * 5/16},
        'K-W(CN)8': {'b0':  1.03200 * 5/8, 'b1': 18.49000 * 5/8, 'Cphi': -0.49370 * 5/16},
     'MeN-Mo(CN)8': {'b0':  0.93800 * 5/8, 'b1': 15.91000 * 5/8, 'Cphi': -0.33300 * 5/16},
        'Na-P3O10': {'b0':  1.86900 * 3/5, 'b1': 36.10000 * 3/5, 'Cphi': -0.16300 * 3/5**(3/2)},
         'K-P3O10': {'b0':  1.93900 * 3/5, 'b1': 39.64000 * 3/5, 'Cphi': -0.10550 * 3/5**(3/2)},
    }
